@@ -23,7 +23,7 @@ Digital Loggers Web Power Switch management
  Author: Dwight Hubbard d@dhub.me
 """
 """
-Copyright (c) 2009, Dwight Hubbard
+Copyright (c) 2009,2010,2011, 2012, Dwight Hubbard
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -54,7 +54,9 @@ import time
 import optparse
 import base64
 import json
+import urllib
 import urllib2
+import multiprocessing
 
 # External modules
 import BeautifulSoup
@@ -65,6 +67,45 @@ CYCLETIME=3
 CONFIG_DEFAULTS={'timeout':TIMEOUT,'cycletime':CYCLETIME,'userid':'admin','password':'4321','hostname':'192.168.0.100'}
 CONFIG_FILE=os.path.expanduser('~/.dlipower.conf')
 
+def _call_it(params):
+    instance, name, args=params
+    "indirect caller for instance methods and multiprocessing"    
+    kwargs = {}
+    return getattr(instance, name)(*args, **kwargs)
+
+def _block_to_list(block):
+  """ Convert a range block into a numeric list 
+      input "1-3,17,19-20"
+      output=[1,2,3,17,19,20]
+  """
+  block+=','
+  result=[]
+  val=''
+  in_range=False
+  for letter in block:
+    if letter in [',','-']:
+      if in_range:
+        val2=val
+        val2_len=len(val2)
+        #result+=range(int(val1),int(val2)+1)
+        for value in range(int(val1),int(val2)+1):
+          result.append(str(value).zfill(val2_len))
+        val=''
+        val1=None
+        in_range=False
+      else:
+        val1=val
+        val1_len=len(val1)
+        val=''
+      if letter == ',':
+        if val1 != None:
+          result.append(val1.zfill(val1_len))
+      else:
+        in_range=True
+    else:
+      val+=letter
+  return result
+
 class powerswitch:
     """ Powerswitch class to manage the Digital Loggers Web power switch """
     def __init__(self,userid=None,password=None,hostname=None,timeout=None,cycletime=None):
@@ -138,8 +179,6 @@ class powerswitch:
             result = urllib2.urlopen(request,timeout=self.timeout).read()
         except urllib2.URLError:
             return None
-        #except timeout:
-        #    return None
         return result
     def determine_outlet(self,outlet=None):
         """ Get the correct outlet number from the outlet passed in, this
@@ -161,6 +200,11 @@ class powerswitch:
                 if int(plug[0]) == outlet:
                     return plug[1]
         return 'Unknown'
+    def set_outlet_name(self,outlet=0,name="Unknown"):
+        """ Set the name of an outlet """
+        self.determine_outlet(outlet)
+        self.geturl(url='unitnames.cgi?outname%s=%s' % (outlet,urllib.quote(name)))
+        return self.get_outlet_name(outlet) == name
     def off(self,outlet=0):
         """ Turn off a power to an outlet 
             False = Success
@@ -232,9 +276,31 @@ class powerswitch:
                 if plug[0] == outlet:
                     return plug[2]
         return 'Unknown'
-
+                    
+    def command_on_outlets(self,command,outlets):
+        """ 
+        If a single outlet is passed, handle it as a single outlet and 
+        pass back the return code.  Otherwise run the operation on multiple
+        outlets in parallel the return code will be failure if any operation
+        fails.  Operations that return a string will return a list of strings.
+        """
+        if len(outlets) == 1:
+            result=getattr(self,command)(outlets[0])
+            if isinstance(result,bool):
+              return result
+            else:
+              return [result]
+        pool=multiprocessing.Pool(processes=len(outlets))
+        result=[value for value in pool.imap(_call_it,[(self,command, (outlet,)) for outlet in outlets],chunksize=1)]
+        if isinstance(result[0],bool):
+          for value in result:
+            if value:
+              return True    
+          return result[0]
+        return result
+        
 if __name__ == "__main__":
-    usage = "usage: %prog [options] [status|on|off|cycle|get_outlet_name] [arg]"
+    usage = "usage: %prog [options] [status|on|off|cycle|get_outlet_name|set_outlet_name] [range|arg]"
     parser = optparse.OptionParser(usage=usage)
     parser.add_option('--hostname',dest='hostname',default=None,help="hostname/ip of the power switch (default %default)")
     parser.add_option('--timeout',dest='timeout',default=None,help="Timeout for value for power switch communication (default %default)")
@@ -249,22 +315,26 @@ if __name__ == "__main__":
     if options.save_settings:
         switch.save_configuration()
     if len(args):
-        if len(args) == 2:
-            if args[0].lower() in ['on','poweron']:
-                rc=switch.on(args[1])
+        operation=args[0].lower()
+        range=_block_to_list(','.join(args[1:]))
+        if len(args) > 1:
+            if operation in ['status']:
+                print(','.join(switch.command_on_outlets('status',range)))
+            elif operation in ['on','poweron']:
+                rc=switch.command_on_outlets('on',range)
                 if rc and not options.quiet:
                   print >> sys.stderr,"Power on operation failed"
-                sys.exit(rc)
-            elif args[0].lower() in ['off','poweroff']:
-                rc=switch.off(args[1])
+                sys.exit(rc)                
+            elif operation in ['off','poweroff']:
+                rc=switch.command_on_outlets('off',range)
                 if rc and not options.quiet:
                   print >> sys.stderr,"Power off operation failed"
-            elif args[0].lower() in ['cycle']:
-                sys.exit(switch.cycle(args[1]))
-            elif args[0].lower() in ['status']:
-                print(switch.status(args[1]))
-            elif args[0].lower() in ['get_name','getname','get_outlet_name','getoutletname']:
-                print(switch.get_outlet_name(args[1]))
+            elif operation in ['cycle']:
+                sys.exit(switch.command_on_outlets('cycle',range))
+            elif operation in ['get_name','getname','get_outlet_name','getoutletname']:
+                print(','.join(switch.command_on_outlets('get_outlet_name',range)))
+            elif operation in ['set_name','setname','set_outlet_name','setoutletname']:
+                sys.exit(switch.set_outlet_name(args[1],args[2]))
             else:
                 print("Unknown argument %s" % args[0])
     else:
